/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 3.0
 *
 * The contents of this file are subject to the General Public License
 * 3.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * -- Exchange 2007/2010 Calendar and Tasks Provider.
 * -- For Thunderbird with the Lightning add-on.
 *
 * This work is a combination of the Storage calendar, part of the default Lightning add-on, and 
 * the "Exchange Data Provider for Lightning" add-on currently, october 2011, maintained by Simon Schubert.
 * Primarily made because the "Exchange Data Provider for Lightning" add-on is a continuation 
 * of old code and this one is build up from the ground. It still uses some parts from the 
 * "Exchange Data Provider for Lightning" project.
 *
 * Author: Michel Verbraak (info@1st-setup.nl)
 * Website: http://www.1st-setup.nl/wordpress/?page_id=133
 * email: exchangecalendar@extensions.1st-setup.nl
 *
 *
 * This code uses parts of the Microsoft Exchange Calendar Provider code on which the
 * "Exchange Data Provider for Lightning" was based.
 * The Initial Developer of the Microsoft Exchange Calendar Provider Code is
 *   Andrea Bittau <a.bittau@cs.ucl.ac.uk>, University College London
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * ***** BEGIN LICENSE BLOCK *****/

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;
var Cr = Components.results;
var components = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

Cu.import("resource://1st-setup/ecFunctions.js");

var EXPORTED_SYMBOLS = ["ExchangeRequest","nsSoap","nsTypes","nsMessages", "nsAutodiscoverResponse", "xml_tag", "getEWSServerVersion"];

var xml_tag = '<?xml version="1.0" encoding="utf-8"?>\n';
var nsSoap = new Namespace("nsSoap", "http://schemas.xmlsoap.org/soap/envelope/");
var nsTypes = new Namespace("nsTypes", "http://schemas.microsoft.com/exchange/services/2006/types");
var nsMessages = new Namespace("nsMessages", "http://schemas.microsoft.com/exchange/services/2006/messages");

var nsAutodiscoverResponse = new Namespace("nsAutodiscoverResponse", "http://schemas.microsoft.com/exchange/autodiscover/outlook/responseschema/2006a");

var gExchangeRequestVersion = "0.1";

var gEWSServerVersion = {};

function getEWSServerVersion(aURL)
{
	
	//return "Exchange2007_SP1";
	if ((aURL) && (gEWSServerVersion[aURL])) {
		return gEWSServerVersion[aURL];
	}

	return "Exchange2007_SP1";
}


function ExchangeRequest(aArgument, aCbOk, aCbError, aListener)
{
	this.mData = "";
	this.mArgument = aArgument;
	this.mCbOk   = aCbOk;
	this.mCbError   = aCbError;
	this.retries = 0;
	this.urllist = [];
	this.currentUrl = "";
	this.listener = aListener;
	this.e4x = true;
	this.retryCount = 0;

	this.mAuthFail = 0;
	this.xmlReq = null;
	this.shutdown = false;
	this.badCert = false;
	this.badCertCount = 0;

	this.uuid = exchWebService.commonFunctions.getUUID();

	this.debug = false;

}

ExchangeRequest.prototype = {
	ER_ERROR_EMPTY_FOLDERPATH: -2,
	ER_ERROR_INVALID_URL: -6, 	// "No url to send request to."
	ER_ERROR_RESPONS_NOT_VALID: -7, // "Respons does not contain expected field"
	ER_ERROR_SOAP_ERROR: -8,	// "Error on creating item:"+responseCode
	ER_ERROR_CREATING_ITEM_UNKNOWN: -13, // "Error. Unknown item creation:"+String(aResp)

	ER_ERROR_USER_ABORT_AUTHENTICATION: -20,	// "User aborted authentication credentials"
	ER_ERROR_USER_ABORT_ADD_CERTIFICATE: -30,	// "User aborted adding required certificate"
	ER_ERROR_OPEN_FAILED: -100,	// "Could not connect to specified host:"+err
	ER_ERROR_FROM_SERVER: -101,	// HTTP error from server.
	ER_ERROR_AUTODISCOVER_GET_EWSULR: -200,  // During auto discovery no EWS URL were discoverd in respons.
	ER_ERROR_FINDFOLDER_NO_TOTALITEMSVIEW: -201, // Field totalitemsview missing in soap response.
	ER_ERROR_FINDFOLDER_FOLDERID_DETAILS: -202, // Could not find folderid details in soap response.
	ER_ERROR_FINDFOLDER_MULTIPLE_RESULTS: -203, // Found more than one results in the findfolder soap response.
	ER_ERROR_FINDOCCURRENCES_INVALIDIDMALFORMED: -204, // Found an malformed id during find occurrences.
	ER_ERROR_GETOCCURRENCEINDEX_NOTFOUND: -205,  // Could not found occurrence index.
	ER_ERROR_SOAP_RESPONSECODE_NOTFOUND: -206, // Could not find the responce field in the soap response.
	ER_ERROR_PRIMARY_SMTP_NOTFOUND: -207, // Primary SMTP address could not be found in soap response.
	ER_ERROR_PRIMARY_SMTP_UNKNOWN: -208,  // Unknown error during Primary SMTP check.
	ER_ERROR_UNKNOWN_MEETING_REPSONSE: -209, // Unknown Meeting Response.
	ER_ERROR_SYNCFOLDERITEMS_UNKNOWN: -210, // Unknown error during SyncFolders.
	ER_ERROR_ITEM_UPDATE_UNKNOWN: -211,  // Unknown error during item ipdate.

	ERR_PASSWORD_ERROR: -300, // To many password errors.

	get argument() {
		return this.mArgument;
	},

	get user() 
	{
		return this.argument.user;
	},

	set user(aValue)
	{
		this.argument.user = aValue;
	},

	stopRequest: function _stopRequest()
	{
		this.shutdown = true;
		this.xmlReq.abort();
	},

	sendRequest: function(aData, aUrl)
	{
		if (this.shutdown) {
			return;
		}

//		exchWebService.commonFunctions.LOG(this.uuid+": sendRequest\n");
		this.mData = aData;
		this.currentUrl = "";

		while ((!aUrl) && (this.urllist.length > 0 )) {
			aUrl = this.urllist[0];
			this.urllist.shift();
		}

		if ((!aUrl) || (aUrl == "") || (aUrl == undefined)) {
			this.fail(this.ER_ERROR_INVALID_URL, "No url to send request to.");
		}

		this.currentUrl = aUrl;

		this.xmlReq = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance();

		this.mXmlReq = this.xmlReq;

		var tmp = this;

		this.xmlReq.addEventListener("load", function(evt) { tmp.reply(evt); }, false);
		this.xmlReq.addEventListener("error", function(evt) { tmp.error(evt); }, false);
		this.xmlReq.addEventListener("abort", function(evt) { tmp.abort(evt); }, false);

		// remove domain part in xmlhttprequest.open call
		//if (this.debug) {
			exchWebService.commonFunctions.LOG(this.uuid+": 1 ExchangeRequest.sendRequest : user="+this.mArgument.user+", url="+this.currentUrl);
		//}
		try {
			this.xmlReq.open("POST", this.currentUrl, true, this.mArgument.user, "");
		}
		catch(err) {
			exchWebService.commonFunctions.LOG(this.uuid+": ERROR on ExchangeRequest.sendRequest to URL:"+this.currentUrl+"."); 
			this.fail(this.ER_ERROR_OPEN_FAILED,"Could not connect to specified host:"+err);
		}

		this.xmlReq.overrideMimeType('text/xml');
		this.xmlReq.setRequestHeader("Content-Type", "text/xml");
		this.xmlReq.setRequestHeader("User-Agent", "extensions.1st-setup.nl/" + gExchangeRequestVersion+"/username="+this.mArgument.user);

		// This is required for NTLM authenticated sessions. Which is default for a default EWS install.
		this.xmlReq.setRequestHeader("Connection", "keep-alive");

		/* set channel notifications for password processing */
		this.xmlReq.channel.notificationCallbacks = new ecnsIAuthPrompt2(this);
		this.xmlReq.channel.loadGroup = null;

		var httpChannel = this.xmlReq.channel.QueryInterface(Ci.nsIHttpChannel);

		// XXX we want to preserve POST across 302 redirects TODO: This might go away because header params are copyied right now.
		httpChannel.redirectionLimit = 0;

		if (this.debug) {
			exchWebService.commonFunctions.LOG(this.uuid+": sendRequest Sending: " + this.mData+"\n");
		}
		this.xmlReq.send(this.mData);
	},

	error: function(evt)
	{
		let xmlReq = this.mXmlReq;

		exchWebService.commonFunctions.LOG(this.uuid+": ExchangeRequest.error :"+evt.type+", readyState:"+xmlReq.readyState+", status:"+xmlReq.status);

		if (this.redirect(evt)) {
			return;
		}

		if ((!this.shutdown) && (this.badCert)) {
			// On this connection a bad certificate was seen. Wait until it resets.
			//this.sendRequest(this.mData, this.currentUrl);
			return;
		}

		exchWebService.commonFunctions.LOG("ecExchangeRequest.error: "+evt.type);
		this.retry();
	},

	abort: function(evt)
	{
		if (evt.type != "abort") {
			exchWebService.commonFunctions.LOG("ecExchangeRequest.abort: "+evt.type);
		}
//		this.fail(-3, "User aborted data transfer.");
	},

	onUserStop: function _onUserStop(aCode, aMsg)
	{
		this.mXmlReq.abort();
		this.fail(aCode, aMsg);
	},

	redirect: function(evt)
	{
		let xmlReq = this.mXmlReq;

		if (xmlReq.readyState != 4)
			return false;

		if (xmlReq.status == 302) {
			if (this.debug) {
				exchWebService.commonFunctions.LOG(this.uuid+": ExchangeRequest.redirect :"+evt.type+", readyState:"+xmlReq.readyState+", status:"+xmlReq.status);
			}

			let httpChannel = xmlReq.channel.QueryInterface(Ci.nsIHttpChannel);
			let loc = httpChannel.getResponseHeader("Location");

			exchWebService.commonFunctions.LOG(this.uuid+": Redirect: " + loc + "\n");

                        // XXX pheer loops.
                        xmlReq.abort();
                        this.sendRequest(this.mData, loc);

			return true;
		}

		return false;
	},

	reply:function(evt) 
	{
		let xmlReq = this.mXmlReq;

		if (this.debug) {
			exchWebService.commonFunctions.LOG(this.uuid+": ExchangeRequest.reply :"+evt.type+", readyState:"+xmlReq.readyState+", status:"+xmlReq.status);
			exchWebService.commonFunctions.LOG(this.uuid+": ExchangeRequest.reply :"+xmlReq.responseText);
		}

		if (xmlReq.readyState != 4)
			return;

		if (this.redirect(evt)) {
			return;
		}

/*		if (xmlReq.status == 302) {
			let httpChannel = xmlReq.channel.QueryInterface(Ci.nsIHttpChannel);
			let loc = httpChannel.getResponseHeader("Location");

			exchWebService.commonFunctions.LOG(this.uuid+": Redirect: " + loc + "\n");

                        // XXX pheer loops.
                        xmlReq.abort();
                        this.sendRequest(this.mData, loc);

			return;
		}*/

		if (this.isConnError()) {
			return;
		}

		var xml = xmlReq.responseText; // bug 270553
		xml = xml.replace(/^<\?xml\s+version\s*=\s*(?:"[^"]+"|'[^']+')[^?]*\?>/, ""); // bug 336551

		xml = xml.replace(/naaimachine/,"Castle on Tue/&#x10;Thur ; Austin M/W/F [eom]");
		xml = xml.replace(/&#x10;/g, ""); // BUG 61 remove hexadecimal code 0x10. It will fail in xml conversion.


		try {
			xml = new XML(xml);
		}
		catch(er) {
			exchWebService.commonFunctions.LOG("XML conversion error 1: "+ er);
			// BUG 61 Convert all Hex codes to decimal
			xml = xml.replace(/&#x([0123456789ABCDEF][0123456789ABCDEF]?);/g, function(str, ent) { return "&#"+parseInt(ent,16)+";"; }); 
			try {
				xml = new XML(xml);
			}
			catch(er2) {
				exchWebService.commonFunctions.LOG("XML conversion error 2: "+ er);
				 // BUG 61 remove all decimal codes below character 32 (space) 
				xml = xml.replace(/&#([0123456789]?);/g, function(str, ent) parseInt(ent,10) < 32 ? "" : "&#"+parseInt(ent,10)+";" );
				try {
					xml = new XML(xml);
				}
				catch (er3) {
					exchWebService.commonFunctions.LOG("FATAL XML conversion error 3: "+ er3);
					exchWebService.commonFunctions.LOG("          String which fails: "+ xml);
					throw "ecExchangeRequest.reply new XML() FATAL:"+er3;
				}
			}
		}

		this.mAuthFail = 0;
		this.mRunning  = false;

		var resp;
		if (this.e4x) {
			resp = xml;
		} else {
			resp = xmlReq.responseXML;
		}

		if (this.mCbOk) {
			// Try to get server version and store it.
			try {
				var serverVersion = resp.nsSoap::Header.nsTypes::ServerVersionInfo;
				if (serverVersion) {
					gEWSServerVersion[this.currentUrl] = serverVersion.@Version;
				}
			}
			catch(err) { }

			this.retryCount = 0;
			this.mCbOk(this, resp);
		}

	},

	retryCurrentUrl: function()
	{
		this.sendRequest(this.mData, this.currentUrl);
	},

	retry: function() 
	{
                let xmlReq = this.mXmlReq;
		xmlReq.abort();

		// Loop through the urllist
		try {
			this.sendRequest(this.mData);
			return true;
		}
		catch(err) {
		}

		this.fail(this.ER_ERROR_INVALID_URL, "No url to send request to.");
		xmlReq.abort();
		return true;
	},

        isConnError: function()
        {
                let xmlReq = this.mXmlReq;

                if (xmlReq.status == 0) {
                        return this.retry();
                }

                if (xmlReq.status != 200) {
                        if (xmlReq.status == 401) {
                                exchWebService.commonFunctions.LOG(this.uuid+": isConnError req.status=401:"+xmlReq.responseText+"\n");

				if (this.urllist.length > 0) {
					this.retry();
					return false;
				}

 				this.fail(xmlReq.status, 'HTTP Response 401: Unauthorized:'+xmlReq.responseText);
                        	return true;
                        }

			this.retry();
                        // XXX parse it
 			//this.fail(this.ER_ERROR_FROM_SERVER, xmlReq.responseText);
                        return true;
                }

                return false;
        },

	fail: function(aCode, aMsg)
	{
		if (this.mCbError) {
			this.mCbError(this, aCode, aMsg);
		}
	},

	makeSoapMessage: function erMakeSoapMessage(aReq)
	{
		var msg = <nsSoap:Envelope xmlns:nsSoap={nsSoap} xmlns:nsTypes={nsTypes} xmlns:nsMessages={nsMessages}/>;

		if (this.mArgument.ServerVersion) {
			msg.nsSoap::Header.nsTypes::RequestServerVersion.@Version = this.mArgument.ServerVersion; 
		}
		else {
			msg.nsSoap::Header.nsTypes::RequestServerVersion.@Version = getEWSServerVersion(); 
		}
		msg.nsSoap::Body.request = aReq;

		return xml_tag + String(msg);
	},

	passwordError: function erPasswordError(aMsg)
	{
		this.fail(this.ERR_PASSWORD_ERROR, aMsg);
	},


};

var ecPasswordErrorList = {};

function ecnsIAuthPrompt2(aExchangeRequest)
{
	this.exchangeRequest = aExchangeRequest;
	this.uuid = exchWebService.commonFunctions.getUUID();
	this.callback = null;
	this.context = null;
	this.level = null;
	this.authInfo = null;
	this.channel = null;
	this.trycount = 0;

	this.username = null;
	this.password = null;
	this.URL = null;
	
	this.timer = Cc["@mozilla.org/timer;1"]
					.createInstance(Ci.nsITimer);
}

ecnsIAuthPrompt2.prototype = {

	QueryInterface: XPCOMUtils.generateQI([Ci.nsIAuthPrompt2, Ci.nsIBadCertListener2, Ci.nsIProgressEventSink, 
						Ci.nsISecureBrowserUI, Ci.nsIDocShellTreeItem, Ci.nsIAuthPromptProvider]),

	getInterface: function(iid)
	{
		if (iid.equals(Ci.nsIAuthPrompt2)) {    // id == 651395eb-8612-4876-8ac0-a88d4dce9e1e
			return this;
		} 

		if (iid.equals(Ci.nsIBadCertListener2)) {
			if (!this.exchangeRequest.badCert) {
	        		return this;
			}
		} 

		if (iid.equals(Ci.nsIProgressEventSink)) {   // iid == d974c99e-4148-4df9-8d98-de834a2f6462
        		return this;
		} 

		if (iid.equals(Ci.nsISecureBrowserUI)) {   // iid == 081e31e0-a144-11d3-8c7c-00609792278c
        		return this;
		} 

		if (iid.equals(Ci.nsIDocShellTreeItem)) {   // iid == 09b54ec1-d98a-49a9-bc95-3219e8b55089
        		return Cr.NS_NOINTERFACE;
		} 

		if (iid.equals(Ci.nsIAuthPromptProvider)) {   // iid == bd9dc0fa-68ce-47d0-8859-6418c2ae8576
        		return this;
		} 

		exchWebService.commonFunctions.LOG("  >>>>>>>>>>> MAIL THIS LINE TO exchangecalendar@extensions.1st-setup.nl: ecnsIAuthPrompt2.getInterface("+iid+")");
		throw Cr.NS_NOINTERFACE;
	},

	// nsIBadCertListener2
	notifyCertProblem: function cBCL_notifyCertProblem(socketInfo, status, targetSite) 
	{
		if (!status) {
			return true;
		}

		this.exchangeRequest.badCert = true;

		// Unfortunately we can't pass js objects using the window watcher, so
		// we'll just take the first available calendar window. We also need to
		// do this on a timer so that the modal window doesn't block the
		// network request.
		let wm = Cc["@mozilla.org/appshell/window-mediator;1"]
      	                    .getService(Ci.nsIWindowMediator);
		let calWindow = wm.getMostRecentWindow("mail:3pane") ;

		let timerCallback = {
			exchangeRequest: this.exchangeRequest,
			notify: function(timer) {
				let params = { exceptionAdded: false,
						prefetchCert: true,
						location: targetSite };
				calWindow.openDialog("chrome://pippki/content/exceptionDialog.xul",
						"",
						"chrome,centerscreen,modal",
						params);

				if (params.exceptionAdded) {
					this.exchangeRequest.badCert = false;
					this.exchangeRequest.retryCurrentUrl();
				}
				else {
					this.exchangeRequest.onUserStop(this.exchangeRequest.ER_ERROR_USER_ABORT_ADD_CERTIFICATE, "User did not add needed certificate.");
				}
				} // function(timer)
			};

		let timer = Components.classes["@mozilla.org/timer;1"]
				.createInstance(Components.interfaces.nsITimer);
		timer.initWithCallback(timerCallback, 0,
				Components.interfaces.nsITimer.TYPE_ONE_SHOT);
		return true;
	},

	// nsIProgressEventSink
	onProgress: function _nsIProgressEventSink_onProgress(aRequest, aContext, aProgress, aProgressMax)
	{
		exchWebService.commonFunctions.LOG("  --- ecnsIAuthPrompt2.onProgress:"+aProgress+" of "+aProgressMax);
	},

	// nsIProgressEventSink
	onStatus: function _nsIProgressEventSink_onStatus(aRequest, aContext, aStatus, aStatusArg)
	{
		exchWebService.commonFunctions.LOG("  --- ecnsIAuthPrompt2.onStatus:"+aStatus+" of "+aStatusArg);
	},

	// nsISecureBrowserUI
	// void init(in nsIDOMWindow window);
	init: function _nsISecureBrowserUI_init(window)
	{
		exchWebService.commonFunctions.LOG("  --- ecnsIAuthPrompt2.init:");
		this.nsISecureBrowserUI_Window = window;
	},

	// nsISecureBrowserUI
	//readonly attribute unsigned long state;
	get state()
	{
		exchWebService.commonFunctions.LOG("  --- ecnsIAuthPrompt2.state:");
	},

	// nsISecureBrowserUI
	//readonly attribute AString tooltipText;
	get tooltipText()
	{
		exchWebService.commonFunctions.LOG("  --- ecnsIAuthPrompt2.tooltipText:");
		return "ecnsIAuthPrompt2.tooltipText";
	},
	
	// nsIAuthPromptProvider
	//void getAuthPrompt(in PRUint32 aPromptReason, in nsIIDRef iid, [iid_is(iid),retval] out nsQIResult result);
	getAuthPrompt: function _nsIAuthPromptProvider_getAuthPrompt(aPromptReason, iid)
	{
		if (iid.equals(Ci.nsIAuthPrompt2)) {    // id == 651395eb-8612-4876-8ac0-a88d4dce9e1e
			return this;
		} 

		exchWebService.commonFunctions.LOG("  --- ecnsIAuthPrompt2.getAuthPrompt:aPromptReason:"+aPromptReason+", iid:"+iid);
		exchWebService.commonFunctions.LOG("  >>>>>>>>>>> MAIL THIS LINE TO exchangecalendar@extensions.1st-setup.nl: ecnsIAuthPrompt2.getAuthPrompt("+iid+")");
  
		return null;
	},

	//void cancel(in nsresult aReason);
	cancel: function _cancel(aReason)
	{
		if (this.callback) {
			this.callback.onAuthCancelled(this.context, false);
		}
	},

	notify: function _nsIAuthPrompt2_timercb() {
		if (this.callback) {
			var password = this.getPassword();
			if (password) {
				if (!(this.authInfo.flags & Ci.nsIAuthInformation.ONLY_PASSWORD)) {
					if (this.authInfo.flags & Ci.nsIAuthInformation.NEED_DOMAIN) {
						if (this.username.indexOf("\\") > -1) {
							this.authInfo.domain = this.username.substr(0,this.username.indexOf("\\"));
							this.authInfo.username = this.username.substr(this.username.indexOf("\\")+1);
						}
						else {
							this.authInfo.domain = "";
							this.authInfo.username = this.username;
						}
					}
					else {
						this.authInfo.username = this.username;
					}
				}
				this.authInfo.password = password;
				//this.logInfo(" USING password for connection:["+password+"]");
				try {
					this.callback.onAuthAvailable(this.context, this.authInfo);
				}
				catch(err) {
					this.logInfo("ecnsIAuthPrompt2.notify ERROR:"+err);
					this.callback.onAuthCancelled(this.context, false);
				}
			}
			else {
				this.callback.onAuthCancelled(this.context, true);
				this.exchangeRequest.onUserStop(this.exchangeRequest.ER_ERROR_USER_ABORT_AUTHENTICATION, "User canceled entering authentication details.");
			}
		}
	},

	// nsIAuthPrompt2
	//nsICancelable asyncPromptAuth(in nsIChannel aChannel, in nsIAuthPromptCallback aCallback, in nsISupports aContext, in PRUint32 level, in nsIAuthInformation authInfo);
	asyncPromptAuth: function _asyncPromptAuth(aChannel, aCallback, aContext, level, authInfo)
	{
		this.username = this.exchangeRequest.mArgument.user;
		this.password = null;

		this.URL = decodeURI(aChannel.URI.prePath+aChannel.URI.path);;

		exchWebService.commonFunctions.LOG("asyncPromptAuth: aChannel.URI="+this.URL+", username="+this.username+", password="+this.password);

		this.channel = aChannel;
		this.callback = aCallback;
		this.context = aContext;
		this.level = level;
		this.authInfo = authInfo;
		this.trycount++;
		if ((ecPasswordErrorList[this.URL]) && (ecPasswordErrorList[this.URL] > 3)) {
			exchWebService.commonFunctions.LOG("ecnsIAuthPrompt2.asyncPromptAuth reset count to big.");
			this.exchangeRequest.passwordError("ecnsIAuthPrompt2.asyncPromptAuth reset count to big.");
			return null;
		}

		if (this.trycount > 4) {
			if (!this.resetPassword()) {
				exchWebService.commonFunctions.LOG("ecnsIAuthPrompt2.asyncPromptAuth COULD NOT RESET PASSWORD");
			}
			else {
				this.trycount = 0;
			}
		}
		this.timer.initWithCallback(this, 5, Ci.nsITimer.TYPE_ONE_SHOT);
		return this;
	},

	// nsIAuthPrompt2
	//boolean promptAuth(in nsIChannel aChannel, in PRUint32 level, in nsIAuthInformation authInfo);
	promptAuth: function _promptAuth(aChannel, level, authInfo)
	{
		this.username = this.exchangeRequest.mArgument.user;
		this.password = null;

		this.URL = decodeURI(aChannel.URI.prePath+aChannel.URI.path);;

		exchWebService.commonFunctions.LOG("asyncPromptAuth: aChannel.URI="+this.URL+", username="+this.username+", password="+this.password);

		var password = this.getPassword();
		if (password) {
			if (!(authInfo.flags & Ci.nsIAuthInformation.ONLY_PASSWORD)) {
				if (authInfo.flags & Ci.nsIAuthInformation.NEED_DOMAIN) {
					if (this.username.indexOf("\\") > -1) {
						authInfo.domain = this.username.substr(0,this.username.indexOf("\\"));
						authInfo.username = this.username.substr(this.username.indexOf("\\")+1);
					}
					else {
						authInfo.domain = "";
						authInfo.username = this.username;
					}
				}
				else {
					authInfo.username = this.username;
				}
			}
			authInfo.password = password;
			return true;
		}
		else {
			this.exchangeRequest.onUserStop();
			return false;
		}
		return false;
	},

	getPassword: function _getPassword()
	{
                var username = { value: this.username };

		exchWebService.commonFunctions.LOG("getPassword for user:"+username.value+", server url:"+this.URL);

                var password = { value: "" };
		if (this.password) {
	                password = { value: this.password };
		}
                var persist  = { value: false };
		var title = "Microsoft Exchange EWS";
		var realm = this.URL;

		var got = this.passwordManagerGet(username.value, password, realm, title);

		if (got) {
			ecPasswordErrorList[this.URL] = 0;
			//this.logInfo(" USING password from passwordmanager:["+password.value+"]");
			return password.value;
		}

		var ok = this.getCredentials(title, realm, username, password, persist);

		if (!ok) {
			if (ecPasswordErrorList[this.URL]) {
				ecPasswordErrorList[this.URL] = ecPasswordErrorList[this.URL] + 1;
			}
			else {
				ecPasswordErrorList[this.URL] = 1;
			}
			return null;
		}

		ecPasswordErrorList[this.URL] = 0;
		this.exchangeRequest.user = username.value;
		this.username = username.value;

		if (persist.value) {
			this.passwordManagerSave(username.value, password.value, realm, title);
		}

		//this.logInfo(" USING password from userinput:["+password.value+"]");
		return password.value;
	},

	resetPassword: function()
	{
		if (!this.username) {
			return false;
		}

		var title = "Microsoft Exchange EWS";
		var realm = this.URL;
		this.passwordManagerRemove(this.username, realm, title);
		return true;
	},

	logInfo: function _logInfo(aMsg)
	{
		exchWebService.commonFunctions.LOG(this.uuid+": "+aMsg);
	},

	// Following functions taken from calAuthUtils.jsm from Lightning

    /**
     * Tries to get the username/password combination of a specific calendar name
     * from the password manager or asks the user.
     *
     * @param   in aTitle           The dialog title.
     * @param   in aRealm    The calendar name or url to look up. Can be null.
     * @param   inout aUsername     The username that belongs to the calendar.
     * @param   inout aPassword     The password that belongs to the calendar.
     * @param   inout aSavePassword Should the password be saved?
     * @param   in aFixedUsername   Whether the user name is fixed or editable
     * @return  Could a password be retrieved?
     */
    getCredentials: function calGetCredentials(aTitle,
                                               aRealm,
                                               aUsername,
                                               aPassword,
                                               aSavePassword,
                                               aFixedUsername) {

        if (typeof aUsername != "object" ||
            typeof aPassword != "object" ||
            typeof aSavePassword != "object") {
            throw new Components.Exception("", Cr.NS_ERROR_XPC_NEED_OUT_OBJECT);
        }

        let watcher = Cc["@mozilla.org/embedcomp/window-watcher;1"]
                                .getService(Ci.nsIWindowWatcher);
        let prompter = watcher.getNewPrompter(null);

        // Only show the save password box if we are supposed to.
        let savepassword = exchWebService.commonFunctions.getString("passwordmgr", "rememberPassword", null, "passwordmgr");

        let aText;
        if (aFixedUsername) {
            aText = exchWebService.commonFunctions.getString("commonDialogs", "EnterPasswordFor", [aUsername.value, aRealm], "global");
            return prompter.promptPassword(aTitle,
                                           aText,
                                           aPassword,
                                           savepassword,
                                           aSavePassword);
        } else {
            aText = exchWebService.commonFunctions.getString("commonDialogs", "EnterUserPasswordFor", [aRealm], "global");
            return prompter.promptUsernameAndPassword(aTitle,
                                                      aText,
                                                      aUsername,
                                                      aPassword,
                                                      savepassword,
                                                      aSavePassword);
        }
    },

	getHostname: function _getHostname(aName)
	{
		var minLength = 7;
		if (aName.indexOf('https://') > -1) {
			minLength = 8;
		}

		while ((minLength < aName.length) && (aName.substr(minLength,1) != "/")) {
			minLength++;
		}

		if (minLength < aName.length) {
			return aName.substr(0, minLength);
		}

		return aName;
	},

    /**
     * Helper to insert/update an entry to the password manager.
     *
     * @param aUserName     The username
     * @param aPassword     The corresponding password
     * @param aHostName     The corresponding hostname
     * @param aRealm        The password realm (unused on branch)
     */
    passwordManagerSave: function calPasswordManagerSave(aUsername, aPassword, aHostName, aRealm) {
        exchWebService.commonFunctions.ASSERT(aUsername);
        exchWebService.commonFunctions.ASSERT(aPassword);

	var tmpHostname = aHostName; //this.getHostname(aHostName);

        try {
            let loginManager = Cc["@mozilla.org/login-manager;1"]
                                         .getService(Ci.nsILoginManager);
            let logins = loginManager.findLogins({}, tmpHostname, null, aRealm);

            let newLoginInfo = Cc["@mozilla.org/login-manager/loginInfo;1"]
                                         .createInstance(Ci.nsILoginInfo);
            newLoginInfo.init(tmpHostname, null, aRealm, aUsername, aPassword, "", "");
            if (logins.length > 0) {
			var modified = false;
			for each (let loginInfo in logins) {
				if (loginInfo.username == aUsername) {
					exchWebService.commonFunctions.LOG("Login credentials updated:username="+aUsername+", aHostname="+aHostName+", aRealm="+aRealm);
					loginManager.modifyLogin(loginInfo, newLoginInfo);
					modified = true;
	                    		break;
				}
			}
			if (!modified) {
				exchWebService.commonFunctions.LOG("Login credentials saved:username="+aUsername+", aHostname="+aHostName+", aRealm="+aRealm);
				loginManager.addLogin(newLoginInfo);
			}
            } else {
		exchWebService.commonFunctions.LOG("Login credentials saved:username="+aUsername+", aHostname="+aHostName+", aRealm="+aRealm);
                loginManager.addLogin(newLoginInfo);
            }
        } catch (exc) {
            exchWebService.commonFunctions.ASSERT(false, exc);
        }
    },

    /**
     * Helper to retrieve an entry from the password manager.
     *
     * @param in  aUsername     The username to search
     * @param out aPassword     The corresponding password
     * @param aHostName         The corresponding hostname
     * @param aRealm            The password realm (unused on branch)
     * @return                  Does an entry exist in the password manager
     */
    passwordManagerGet: function calPasswordManagerGet(aUsername, aPassword, aHostName, aRealm) {
        exchWebService.commonFunctions.ASSERT(aUsername);

	exchWebService.commonFunctions.LOG("passwordManagerGet: username="+aUsername+", aHostname="+aHostName+", aRealm="+aRealm);
	var tmpHostname = aHostName; //this.getHostname(aHostName);

        if (typeof aPassword != "object") {
            throw new Components.Exception("", Cr.NS_ERROR_XPC_NEED_OUT_OBJECT);
        }

        try {
            let loginManager = Cc["@mozilla.org/login-manager;1"]
                                         .getService(Ci.nsILoginManager);

            let logins = loginManager.findLogins({}, tmpHostname, null, aRealm);
            for each (let loginInfo in logins) {
                if (loginInfo.username == aUsername) {
			exchWebService.commonFunctions.LOG("passwordManagerGet found password for: username="+aUsername+", aHostname="+aHostName+", aRealm="+aRealm);
                    aPassword.value = loginInfo.password;
                    return true;
                }
            }
        } catch (exc) {
            exchWebService.commonFunctions.ASSERT(false, exc);
        }
        return false;
    },

    /**
     * Helper to remove an entry from the password manager
     *
     * @param aUsername     The username to remove.
     * @param aHostName     The corresponding hostname
     * @param aRealm        The password realm (unused on branch)
     * @return              Could the user be removed?
     */
    passwordManagerRemove: function calPasswordManagerRemove(aUsername, aHostName, aRealm) {
        exchWebService.commonFunctions.ASSERT(aUsername);

	exchWebService.commonFunctions.LOG("passwordManagerRemove: username="+aUsername+", aHostname="+aHostName+", aRealm="+aRealm);
 	var tmpHostname = aHostName; //this.getHostname(aHostName);

       try {
            let loginManager = Cc["@mozilla.org/login-manager;1"]
                                         .getService(Ci.nsILoginManager);
            let logins = loginManager.findLogins({}, tmpHostname, null, aRealm);
            for each (let loginInfo in logins) {
                if (loginInfo.username == aUsername) {
                    loginManager.removeLogin(loginInfo);
                    return true;
                }
            }
        } catch (exc) {
        }
        return false;
    }
	// End funcions from calAuthUtils.jsm
}
