/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 3.0
 *
 * The contents of this file are subject to the General Public License
 * 3.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * -- Exchange 2007/2010 Calendar and Tasks Provider.
 * -- For Thunderbird with the Lightning add-on.
 *
 * This work is a combination of the Storage calendar, part of the default Lightning add-on, and 
 * the "Exchange Data Provider for Lightning" add-on currently, october 2011, maintained by Simon Schubert.
 * Primarily made because the "Exchange Data Provider for Lightning" add-on is a continuation 
 * of old code and this one is build up from the ground. It still uses some parts from the 
 * "Exchange Data Provider for Lightning" project.
 *
 * Author: Michel Verbraak (info@1st-setup.nl)
 * Website: http://www.1st-setup.nl/wordpress/?page_id=133
 * email: exchangecalendar@extensions.1st-setup.nl
 *
 *
 * This code uses parts of the Microsoft Exchange Calendar Provider code on which the
 * "Exchange Data Provider for Lightning" was based.
 * The Initial Developer of the Microsoft Exchange Calendar Provider Code is
 *   Andrea Bittau <a.bittau@cs.ucl.ac.uk>, University College London
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * ***** BEGIN LICENSE BLOCK *****/

var Cu = Components.utils;

Cu.import("resource://1st-setup/ecFunctions.js");
Cu.import("resource://exchangecalendar/ecExchangeRequest.js");
Cu.import("resource://exchangecalendar/soapFunctions.js");

var EXPORTED_SYMBOLS = ["erGetItemsRequest"];

function erGetItemsRequest(aArgument, aCbOk, aCbError, aListener)
{
	this.mCbOk = aCbOk;
	this.mCbError = aCbError;

	var self = this;

	this.parent = new ExchangeRequest(aArgument, 
		function(aExchangeRequest, aResp) { self.onSendOk(aExchangeRequest, aResp);},
		function(aExchangeRequest, aCode, aMsg) { self.onSendError(aExchangeRequest, aCode, aMsg);},
		aListener);

	this.argument = aArgument;
	this.serverUrl = aArgument.serverUrl;
	this.listener = aListener;
	this.ids = aArgument.ids;
	this.counter = 0;

	this.isRunning = true;
	this.execute();
}

erGetItemsRequest.prototype = {

	execute: function _execute()
	{
//		exchWebService.commonFunctions.LOG("erGetTaskItemsRequest.execute\n");

		var req = <nsMessages:GetItem xmlns:nsMessages={nsMessages} xmlns:nsTypes={nsTypes}/>;

		req.nsMessages::ItemShape.nsTypes::BaseShape = "AllProperties";
		req.nsMessages::ItemShape.nsTypes::BodyType = 'Text';

		if (this.argument.GUID) {
			req.nsMessages::ItemShape.nsTypes::AdditionalProperties.content = <>
			    	<nsTypes:ExtendedFieldURI PropertySetId={this.argument.GUID} PropertyName="alarmLastAck" PropertyType="SystemTime" xmlns:nsTypes={nsTypes}/>
			    	<nsTypes:ExtendedFieldURI PropertySetId={this.argument.GUID} PropertyName="lastLightningModified" PropertyType="SystemTime" xmlns:nsTypes={nsTypes}/>
			    	<nsTypes:ExtendedFieldURI PropertySetId={this.argument.GUID} PropertyName="X-MOZ-SNOOZE-TIME" PropertyType="String" xmlns:nsTypes={nsTypes}/>
			    	<nsTypes:ExtendedFieldURI PropertySetId={this.argument.GUID} PropertyName="X-MOZ-SNOOZE-TIME-" PropertyType="String" xmlns:nsTypes={nsTypes}/>
			    	<nsTypes:ExtendedFieldURI DistinguishedPropertySetId="Common" PropertyId="34144" PropertyType="SystemTime" xmlns:nsTypes={nsTypes}/>
			    </>;
		}

		var itemids = <nsMessages:ItemIds xmlns:nsMessages={nsMessages}/>;
		for each (var item in this.ids) {
			itemids.nsTypes::ItemId += <nsTypes:ItemId Id={item.Id} ChangeKey={item.ChangeKey} xmlns:nsTypes={nsTypes} />;
		}

		req.appendChild(itemids);

                this.parent.sendRequest(this.parent.makeSoapMessage(req), this.serverUrl);
	},

	onSendOk: function _onSendOk(aExchangeRequest, aResp)
	{
		//exchWebService.commonFunctions.LOG("erGetTaskItemsRequest.onSendOk: "+String(aResp)+"\n");
		var items = [];
		for each (var e in aResp..nsMessages::Items.*) {
			items.push(e);
		}
	
		if (this.mCbOk) {
			this.mCbOk(this, items);
		}
		this.isRunning = false;
	},

	onSendError: function _onSendError(aExchangeRequest, aCode, aMsg)
	{
		this.isRunning = false;
		if (this.mCbError) {
			this.mCbError(this, aCode, aMsg);
		}
	},
};


